#include <cmath>
#include "ExampleMotionPlatform.hpp"
#include "PluginStatusCodes.hpp"

ExampleMotionPlatform::ExampleMotionPlatform(  //
    const varjo_PluginUtilityAPI* pluginUtilityAPI,    //
    varjo_PluginUtilitySession pluginUtilitySession)
    : m_pluginUtilityAPI(pluginUtilityAPI)
    , m_pluginUtilitySession(pluginUtilitySession)
{
}

varjo_PluginStatus ExampleMotionPlatform::updatePose(const varjo_DevicePose* pose, varjo_DevicePose* updatedPose)
{  //
    if (pose == nullptr) return varjo_PluginStatus_NullPointer;
    if (updatedPose == nullptr) return varjo_PluginStatus_NullPointer;

    // Rotate pose 180 degrees around z axis

    const double tx = pose->position.x;
    const double ty = pose->position.y;
    const double tz = pose->position.z;

    const double w = pose->rotation.w;
    const double vx = pose->rotation.x;
    const double vy = pose->rotation.y;
    const double vz = pose->rotation.z;

    *updatedPose = *pose;
    const double norm = std::sqrt(w * w + vx * vx + vy * vy + vz * vz);
    if (norm > 1e-6) {
        updatedPose->rotation.w = -vz / norm;
        updatedPose->rotation.x = -vy / norm;
        updatedPose->rotation.y = -vx / norm;
        updatedPose->rotation.z = w / norm;

        updatedPose->position.x = -tx;
        updatedPose->position.y = -ty;
        updatedPose->position.z = tz;
    }

    return varjo_PluginStatus_OK;
}
