// Copyright 2019-2021 Varjo Technologies Oy. All rights reserved.

#ifndef VARJO_PLUGIN_COMMON_H
#define VARJO_PLUGIN_COMMON_H

#include "Varjo_export.h"
#include "Varjo_types.h"
#include "Varjo_version.h"

#if defined __cplusplus
extern "C" {
#endif  // __cplusplus

/**
 * Provides plugin types supported by the plugin library
 *
 * @return a combination of @ref varjo_PluginType flags
 */
VARJO_API varjo_PluginType varjo_GetPluginType();

/**
 * Provides the API version number supported by the plugin.
 * Use @ref VARJO_PLUGIN_API_VERSION macro to indicate supported API version number.
 *
 * @return Varjo plugin API version number.
 */
VARJO_API uint64_t varjo_GetPluginAPIVersion();

/**
 * Provides plugin name
 *
 * @return Null-terminated C-string containing plugin name
 */
VARJO_API const char* varjo_GetPluginName();

/**
 * Provides a string description of a given status code
 *
 * @param pluginStatus - plugin status code
 * @return Null-terminated C-string containing description of a given status code
 */
VARJO_API const char* varjo_GetPluginStatusDescription(varjo_PluginStatus pluginStatus);

/**
 * Initializes plugin
 *
 * @param pluginUtilityAPI - pointer to structure containing utility function pointers
 * @param pluginUtilitySession - handle to pass to @ref varjo_PluginUtilityAPI functions
 * @param context - pointer to write plugin session context
 * @return @ref varjo_PluginStatus_OK if plugin was initialized successfully, other value of
 *         type @ref varjo_PluginStatus otherwise
 */
VARJO_API varjo_PluginStatus varjo_PluginInit(        //
    const varjo_PluginUtilityAPI* pluginUtilityAPI,   //
    varjo_PluginUtilitySession pluginUtilitySession,  //
    varjo_PluginContext* context);

/**
 * Deinitializes plugin
 *
 * @param context - plugin session context acquired from varjo_PluginInit
 * @return @ref varjo_PluginStatus_OK if plugin was shut down successfully, other value of
 *         type @ref varjo_PluginStatus otherwise
 */
VARJO_API varjo_PluginStatus varjo_PluginShutdown(varjo_PluginContext* context);

#if defined __cplusplus
}  // extern "C"
#endif  // __cplusplus

#endif  // VARJO_PLUGIN_COMMON_H
